#define PSclock     2               // Pin:D2
#define PSdata      3               // Pin:D3
#define PSack       4               // Pin:D4
#define PScommand   5               // Pin:D5
#define PSattention     6           // Pin:D6
//order of the data in packet
enum JS {L3,R3,Rx,Ry,Lx,Ly,SEL,SRT};
//button bit mask
enum bt {sel,l3,r3,st};
#include <SPI.h>
#include "nRF24L01.h"
#include "RF24_config.h"
#include "RF24.h"
#include "printf.h"

void initps2();
int deadzone(int data);
//RF24 CE to Pin D9, CSN to Pin D10
RF24 radio(9,10);

// Radio pipe address for communication.
const uint64_t pipe = 0xE8E8F0F0E1LL;
void setup () {
  //set serial baudrate
  Serial.begin (57600);
  printf_begin();
  //initialize radio
  radio.begin();
  //choose address for writting
  radio.openWritingPipe(pipe);
  //flush buff
  radio.stopListening();
  initps2();
  pinMode(8,OUTPUT);  
  pinMode(7,OUTPUT);
  
  pinMode(14, OUTPUT);
  pinMode(15, OUTPUT);  
  pinMode(16, OUTPUT);  
  pinMode(17, OUTPUT);  
}
int data[8];
byte temp;
byte cycleCount = 0;
byte errorCount = 0;
byte reply;

// main program loop:
void loop (){
/////////////read ps2///////////////////////
  digitalWrite(PScommand, HIGH);
  digitalWrite(PSclock, HIGH);
  digitalWrite(PSattention,LOW);

  gameByte(0x01);                          // bite 0. header
  gameByte(0x42);                          // bite 1. header
  gameByte(0x00);                          // bite 2. header
  
  int i;
  for(i=0; i<6; i++)
    //read ps2 controller digital and analog values
    data[i] = gameByte(0x00);
    
  digitalWrite(PScommand, HIGH);           // close communication with PS2 controller
  delayMicroseconds(1);
  digitalWrite(PSattention,HIGH);          // all done.
  temp = data[0];
  
  //bit mask each individual bit
  data[SEL] = temp&0x1;         //select
  data[L3] = (temp&0x2)>>l3;    //l3
  data[R3] = (temp&0x4)>>r3;    //r3
  data[SRT] = (temp&0x8)>>st;    //start
  


  for(i=Rx;i<=Ly;i++)
    //dead zone check
    data[i]=deadzone(data[i]);
    
  //map joysticks to 4 control inputs
  data[Rx] = map(data[Rx],0,255,1300,1750);  //Rx->YAW
  data[Ry] = map(data[Ry],255,0,1000,1750);  //Ry->THROTLE
  data[Lx] = map(data[Lx],20,235,1300,1700);  //Lx->ROLL
  data[Ly] = map(data[Ly],235,20,1300,1700);  //Ly->PITCH
  
  //print data to serial monitor
  printf("%d, %d, %d, %d, %d, %d, %d, %d\n\r",data[SEL],data[L3],data[R3],data[SRT],data[Rx],data[Ry],data[Lx],data[Ly]);
//////////////////transmit////////////////////////
  reply = radio.write(data,sizeof(data));
  if (!reply)
    errorCount++;
  if(++cycleCount == 100)
    {
    cycleCount = 0;
    errorCount = 0; 
    }
  
  for (i = 0; i <= 3; i++)
    {
    if (errorCount >= i + 1)
      digitalWrite(14 + i, true);
    else
      digitalWrite(14 + i, false);
    }
}

// PSx controller communication function.
// send a byte on the command line and receive one on the data line.
// needs Attention pin to have gone low before called to activate controller.
int gameByte(short int command) {
  short int i ; 
  delayMicroseconds(1);
  short int data = 0x00;                                     // clear data variable to save setting low bits later.
  for(i=0;i<8;i++) {
    if(command & _BV(i))
      digitalWrite(PScommand, HIGH);                         // bit bang "command" out on PScommand wire.
    else
      digitalWrite(PScommand, LOW);
    digitalWrite(PSclock, LOW);                             // CLOCK LOW
    delayMicroseconds(20);                                  // wait for output to stabilise
    if((PIND & _BV(PSdata)))
      _SFR_BYTE(data) |= _BV(i);                            // read PSdata pin and store
    digitalWrite(PSclock, HIGH);                             // CLOCK HIGH
  }
  digitalWrite(PScommand,HIGH);
  delayMicroseconds(20);                                                   // wait for ACK to pass.
  return(data);
}
void initps2() {
  // PSx controller I/O pin setup:
  pinMode (PSclock, OUTPUT);                 // clock. output. (blue)
  pinMode (PSdata, INPUT);                  // data. input. (brown)
  digitalWrite (PSdata, HIGH);               //    enable pullup resistor
  pinMode (PSack, INPUT);                  // acknowledge. input. (green)
  digitalWrite (PSack, HIGH);              //    enable pullup resistor
  pinMode (PScommand, OUTPUT);                 // command. output. (orange)
  pinMode (PSattention, OUTPUT);                 // attention. output. (yellow)

  // enable interupts

  sei();


  // this loop continues to put PS2 controller into analouge mode untill the 
  // controller responds with 0x73 in the 2nd byte.  
  // (PS2 controller responds with 0x73 when in analouge mode.)
  // the status LEDs will continue to count upwards untill a controller is found.
  // if everything is working correctly this should happen on the first pass of 
  // this loop but occasionally errors occur and a 2nd or 3rd itteration happen.
  unsigned char chk_ana = 0, cnt = 0;
  while(chk_ana != 0x73){
    // put controller in config mode
    digitalWrite(PScommand, HIGH);
    digitalWrite(PSclock, HIGH);
    digitalWrite(PSattention,LOW);

    gameByte(0x01);
    gameByte(0x43);
    gameByte(0x00);
    gameByte(0x01);
    gameByte(0x00);

    digitalWrite(PScommand, HIGH);
    delay(1);
    digitalWrite(PSattention, HIGH);

    delay(10);

    // put controller in analog mode
    digitalWrite(PScommand, HIGH);
    digitalWrite(PSclock, HIGH);
    digitalWrite(PSattention, LOW);

    gameByte(0x01);
    gameByte(0x44);
    gameByte(0x00);
    gameByte(0x01);
    gameByte(0x03);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);

    digitalWrite(PScommand, HIGH);
    delay(1);
    digitalWrite(PSattention,HIGH);

    delay(10);

    // exit config mode
    digitalWrite(PScommand, HIGH);
    digitalWrite(PSclock, HIGH);
    digitalWrite(PSattention,LOW);

    gameByte(0x01);
    gameByte(0x43);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x5A);
    gameByte(0x5A);
    gameByte(0x5A);
    gameByte(0x5A);
    gameByte(0x5A);

    digitalWrite(PScommand, HIGH);
    delay(1);
    digitalWrite(PSattention, HIGH);

    delay(10);

    // poll controller and check in analouge mode.
    digitalWrite(PScommand, HIGH);
    digitalWrite(PSclock, HIGH);
    digitalWrite(PSattention,LOW);

    gameByte(0x01);
    chk_ana = gameByte(0x42);            // the 2nd byte to be returned from the controller should = 0x73 for "red" analouge controller.
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);
    gameByte(0x00);

    digitalWrite(PScommand, HIGH);
    delay(1);
    digitalWrite(PSattention, HIGH);

    delay(10);


    // keep increasing counter to be dispalyed untill PS2 controller confirms it's in analouge mode.
    Serial.println (cnt++);                     
    if (cnt > 254){ 
      cnt=0;
    }
  }
}
//deadzone around center, +/- 20.
//limit value from 20 to 235
int deadzone(int data){
  //keep value to 127 if within +/- 20
  if(data<=147&data>=107)
    data = 127;
  //if value is greater than 147,decrease by 20
  else if(data>147)
    data = data -20;
  //if value is less than 107, increase by 20
  else if(data<107)
    data = data + 20;
  return data;
}
